/*
Class that convers Belarusian text to/from kyrylica and lacinka

Copyright (C) 2000  Alexander Mikhailian (mikhailian@altern.org)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


/* NB - Process HARD_SIGN - delete it from input files*/


#include "convert.h"
#include "ASCII.h"
#include "CP1251.h"
#include "CP1250.h"


CConvert::CConvert(void)
{
	a_cyr_lcase[0]  = CYRILLIC_SMALL_LETTER_A;
	a_cyr_lcase[1]  = CYRILLIC_SMALL_LETTER_BE;
	a_cyr_lcase[2]  = CYRILLIC_SMALL_LETTER_VE;
	a_cyr_lcase[3]  = CYRILLIC_SMALL_LETTER_GHE;
	a_cyr_lcase[4]  = CYRILLIC_SMALL_LETTER_DE;
	a_cyr_lcase[5]  = CYRILLIC_SMALL_LETTER_IE;
	a_cyr_lcase[6]  = CYRILLIC_SMALL_LETTER_IO;
	a_cyr_lcase[7]  = CYRILLIC_SMALL_LETTER_ZHE;
	a_cyr_lcase[8]  = CYRILLIC_SMALL_LETTER_ZE;
	a_cyr_lcase[9]  = CYRILLIC_SMALL_LETTER_I;
	a_cyr_lcase[10] = CYRILLIC_SMALL_LETTER_BYELORUSSIAN_UKRAINIAN_I;
	a_cyr_lcase[11] = CYRILLIC_SMALL_LETTER_SHORT_I;
	a_cyr_lcase[12] = CYRILLIC_SMALL_LETTER_KA;
	a_cyr_lcase[13] = CYRILLIC_SMALL_LETTER_EL;
	a_cyr_lcase[14] = CYRILLIC_SMALL_LETTER_EM;
	a_cyr_lcase[15] = CYRILLIC_SMALL_LETTER_EN;
	a_cyr_lcase[16] = CYRILLIC_SMALL_LETTER_O;
	a_cyr_lcase[17] = CYRILLIC_SMALL_LETTER_PE;
	a_cyr_lcase[18] = CYRILLIC_SMALL_LETTER_ER;
	a_cyr_lcase[19] = CYRILLIC_SMALL_LETTER_ES;
	a_cyr_lcase[20] = CYRILLIC_SMALL_LETTER_TE;
	a_cyr_lcase[21] = CYRILLIC_SMALL_LETTER_U;
	a_cyr_lcase[22] = CYRILLIC_SMALL_LETTER_SHORT_U;
	a_cyr_lcase[23] = CYRILLIC_SMALL_LETTER_EF;
	a_cyr_lcase[24] = CYRILLIC_SMALL_LETTER_HA;
	a_cyr_lcase[25] = CYRILLIC_SMALL_LETTER_TSE;
	a_cyr_lcase[26] = CYRILLIC_SMALL_LETTER_CHE;
	a_cyr_lcase[27] = CYRILLIC_SMALL_LETTER_SHA;
	a_cyr_lcase[28] = CYRILLIC_SMALL_LETTER_SHCHA;
	a_cyr_lcase[29] = CYRILLIC_SMALL_LETTER_HARD_SIGN;
	a_cyr_lcase[30] = CYRILLIC_SMALL_LETTER_YERU;
	a_cyr_lcase[31] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
	a_cyr_lcase[32] = CYRILLIC_SMALL_LETTER_E;
	a_cyr_lcase[33] = CYRILLIC_SMALL_LETTER_YU;
	a_cyr_lcase[34] = CYRILLIC_SMALL_LETTER_YA;

	a_cyr_ucase[0]  = CYRILLIC_CAPITAL_LETTER_A;
	a_cyr_ucase[1]  = CYRILLIC_CAPITAL_LETTER_BE;
	a_cyr_ucase[2]  = CYRILLIC_CAPITAL_LETTER_VE;
	a_cyr_ucase[3]  = CYRILLIC_CAPITAL_LETTER_GHE;
	a_cyr_ucase[4]  = CYRILLIC_CAPITAL_LETTER_DE;
	a_cyr_ucase[5]  = CYRILLIC_CAPITAL_LETTER_IE;
	a_cyr_ucase[6]  = CYRILLIC_CAPITAL_LETTER_IO;
	a_cyr_ucase[7]  = CYRILLIC_CAPITAL_LETTER_ZHE;
	a_cyr_ucase[8]  = CYRILLIC_CAPITAL_LETTER_ZE;
	a_cyr_ucase[9]  = CYRILLIC_CAPITAL_LETTER_I;
	a_cyr_ucase[10] = CYRILLIC_CAPITAL_LETTER_BYELORUSSIAN_UKRAINIAN_I;
	a_cyr_ucase[11] = CYRILLIC_CAPITAL_LETTER_SHORT_I;
	a_cyr_ucase[12] = CYRILLIC_CAPITAL_LETTER_KA;
	a_cyr_ucase[13] = CYRILLIC_CAPITAL_LETTER_EL;
	a_cyr_ucase[14] = CYRILLIC_CAPITAL_LETTER_EM;
	a_cyr_ucase[15] = CYRILLIC_CAPITAL_LETTER_EN;
	a_cyr_ucase[16] = CYRILLIC_CAPITAL_LETTER_O;
	a_cyr_ucase[17] = CYRILLIC_CAPITAL_LETTER_PE;
	a_cyr_ucase[18] = CYRILLIC_CAPITAL_LETTER_ER;
	a_cyr_ucase[19] = CYRILLIC_CAPITAL_LETTER_ES;
	a_cyr_ucase[20] = CYRILLIC_CAPITAL_LETTER_TE;
	a_cyr_ucase[21] = CYRILLIC_CAPITAL_LETTER_U;
	a_cyr_ucase[22] = CYRILLIC_CAPITAL_LETTER_SHORT_U;
	a_cyr_ucase[23] = CYRILLIC_CAPITAL_LETTER_EF;
	a_cyr_ucase[24] = CYRILLIC_CAPITAL_LETTER_HA;
	a_cyr_ucase[25] = CYRILLIC_CAPITAL_LETTER_TSE;
	a_cyr_ucase[26] = CYRILLIC_CAPITAL_LETTER_CHE;
	a_cyr_ucase[27] = CYRILLIC_CAPITAL_LETTER_SHA;
	a_cyr_ucase[28] = CYRILLIC_CAPITAL_LETTER_SHCHA;
	a_cyr_ucase[29] = CYRILLIC_CAPITAL_LETTER_HARD_SIGN;
	a_cyr_ucase[30] = CYRILLIC_CAPITAL_LETTER_YERU;
	a_cyr_ucase[31] = CYRILLIC_CAPITAL_LETTER_SOFT_SIGN;
	a_cyr_ucase[32] = CYRILLIC_CAPITAL_LETTER_E;
	a_cyr_ucase[33] = CYRILLIC_CAPITAL_LETTER_YU;
	a_cyr_ucase[34] = CYRILLIC_CAPITAL_LETTER_YA;

	a_lat_lcase[0]  = LATIN_SMALL_LETTER_A;
	a_lat_lcase[1]  = LATIN_SMALL_LETTER_B;
	a_lat_lcase[2]  = LATIN_SMALL_LETTER_V;
	a_lat_lcase[3]  = LATIN_SMALL_LETTER_G;
	a_lat_lcase[4]  = LATIN_SMALL_LETTER_D;
	a_lat_lcase[5]  = LATIN_SMALL_LETTER_Z_WITH_CARON;
	a_lat_lcase[6]  = LATIN_SMALL_LETTER_Z_WITH_ACUTE;
	a_lat_lcase[7]  = LATIN_SMALL_LETTER_I;
	a_lat_lcase[8]  = LATIN_SMALL_LETTER_J;
	a_lat_lcase[9] = LATIN_SMALL_LETTER_K;
	a_lat_lcase[10] = LATIN_SMALL_LETTER_L_WITH_STROKE;
	a_lat_lcase[11] = LATIN_SMALL_LETTER_L;
	a_lat_lcase[12] = LATIN_SMALL_LETTER_M;
	a_lat_lcase[13] = LATIN_SMALL_LETTER_N;
	a_lat_lcase[14] = LATIN_SMALL_LETTER_N_WITH_ACUTE;
	a_lat_lcase[15] = LATIN_SMALL_LETTER_O;
	a_lat_lcase[16] = LATIN_SMALL_LETTER_P;
	a_lat_lcase[17] = LATIN_SMALL_LETTER_R;
	a_lat_lcase[18] = LATIN_SMALL_LETTER_S;
	a_lat_lcase[19] = LATIN_SMALL_LETTER_S_WITH_ACUTE;
	a_lat_lcase[20] = LATIN_SMALL_LETTER_T;
	a_lat_lcase[21] = LATIN_SMALL_LETTER_U;
	a_lat_lcase[22] = LATIN_SMALL_LETTER_U_WITH_BREVE; // not available in CP1250
	a_lat_lcase[23] = LATIN_SMALL_LETTER_F;
	a_lat_lcase[24] = LATIN_SMALL_LETTER_H;
	a_lat_lcase[25] = LATIN_SMALL_LETTER_C;
	a_lat_lcase[26] = LATIN_SMALL_LETTER_C_WITH_ACUTE;
	a_lat_lcase[27] = LATIN_SMALL_LETTER_C_WITH_CARON;
	a_lat_lcase[28] = LATIN_SMALL_LETTER_S_WITH_CARON;
	a_lat_lcase[29] = LATIN_SMALL_LETTER_Y;
	a_lat_lcase[30] = LATIN_SMALL_LETTER_E;

	a_lat_ucase[0]  = LATIN_CAPITAL_LETTER_A;
	a_lat_ucase[1]  = LATIN_CAPITAL_LETTER_B;
	a_lat_ucase[2]  = LATIN_CAPITAL_LETTER_V;
	a_lat_ucase[3]  = LATIN_CAPITAL_LETTER_G;
	a_lat_ucase[4]  = LATIN_CAPITAL_LETTER_D;
	a_lat_ucase[5]  = LATIN_CAPITAL_LETTER_Z_WITH_CARON;
	a_lat_ucase[6]  = LATIN_CAPITAL_LETTER_Z_WITH_ACUTE;
	a_lat_ucase[7]  = LATIN_CAPITAL_LETTER_I;
	a_lat_ucase[8]  = LATIN_CAPITAL_LETTER_J; 
	a_lat_ucase[9] =  LATIN_CAPITAL_LETTER_K;
	a_lat_ucase[10] = LATIN_CAPITAL_LETTER_L_WITH_STROKE;
	a_lat_ucase[11] = LATIN_CAPITAL_LETTER_L;
	a_lat_ucase[12] = LATIN_CAPITAL_LETTER_M;
	a_lat_ucase[13] = LATIN_CAPITAL_LETTER_N;
	a_lat_ucase[14] = LATIN_CAPITAL_LETTER_N_WITH_ACUTE;
	a_lat_ucase[15] = LATIN_CAPITAL_LETTER_O;
	a_lat_ucase[16] = LATIN_CAPITAL_LETTER_P;
	a_lat_ucase[17] = LATIN_CAPITAL_LETTER_R;
	a_lat_ucase[18] = LATIN_CAPITAL_LETTER_S;
	a_lat_ucase[19] = LATIN_CAPITAL_LETTER_S_WITH_ACUTE;
	a_lat_ucase[20] = LATIN_CAPITAL_LETTER_T;
	a_lat_ucase[21] = LATIN_CAPITAL_LETTER_U;
	a_lat_ucase[22] = LATIN_CAPITAL_LETTER_U_WITH_BREVE; // not available in CP1250
	a_lat_ucase[23] = LATIN_CAPITAL_LETTER_F;
	a_lat_ucase[24] = LATIN_CAPITAL_LETTER_H;
	a_lat_ucase[25] = LATIN_CAPITAL_LETTER_C;
	a_lat_ucase[26] = LATIN_CAPITAL_LETTER_C_WITH_ACUTE;
	a_lat_ucase[27] = LATIN_CAPITAL_LETTER_C_WITH_CARON;
	a_lat_ucase[28] = LATIN_CAPITAL_LETTER_S_WITH_CARON;
	a_lat_ucase[29] = LATIN_CAPITAL_LETTER_Y;
	a_lat_ucase[30] = LATIN_CAPITAL_LETTER_E;

	a_lat_lvow[0] = LATIN_SMALL_LETTER_A;
	a_lat_lvow[1] = LATIN_SMALL_LETTER_E;
	a_lat_lvow[2] = LATIN_SMALL_LETTER_O;
	a_lat_lvow[3] = LATIN_SMALL_LETTER_I;
	a_lat_lvow[4] = LATIN_SMALL_LETTER_U;
	a_lat_lvow[5] = LATIN_SMALL_LETTER_Y;

	a_cyr_lhrdcontxt[0]  = CYRILLIC_SMALL_LETTER_A;
	a_cyr_lhrdcontxt[1]  = CYRILLIC_SMALL_LETTER_BE;
	a_cyr_lhrdcontxt[2]  = CYRILLIC_SMALL_LETTER_VE;
	a_cyr_lhrdcontxt[3]  = CYRILLIC_SMALL_LETTER_GHE;
	a_cyr_lhrdcontxt[4]  = CYRILLIC_SMALL_LETTER_DE;
	a_cyr_lhrdcontxt[5]  = CYRILLIC_SMALL_LETTER_ZHE;
	a_cyr_lhrdcontxt[6]  = CYRILLIC_SMALL_LETTER_ZE;
	a_cyr_lhrdcontxt[7] = CYRILLIC_SMALL_LETTER_SHORT_I;
	a_cyr_lhrdcontxt[8] = CYRILLIC_SMALL_LETTER_KA;
	a_cyr_lhrdcontxt[9] = CYRILLIC_SMALL_LETTER_EL;
	a_cyr_lhrdcontxt[10] = CYRILLIC_SMALL_LETTER_EM;
	a_cyr_lhrdcontxt[11] = CYRILLIC_SMALL_LETTER_EN;
	a_cyr_lhrdcontxt[12] = CYRILLIC_SMALL_LETTER_O;
	a_cyr_lhrdcontxt[13] = CYRILLIC_SMALL_LETTER_PE;
	a_cyr_lhrdcontxt[14] = CYRILLIC_SMALL_LETTER_ER;
	a_cyr_lhrdcontxt[15] = CYRILLIC_SMALL_LETTER_ES;
	a_cyr_lhrdcontxt[16] = CYRILLIC_SMALL_LETTER_TE;
	a_cyr_lhrdcontxt[17] = CYRILLIC_SMALL_LETTER_U;
	a_cyr_lhrdcontxt[18] = CYRILLIC_SMALL_LETTER_SHORT_U;
	a_cyr_lhrdcontxt[19] = CYRILLIC_SMALL_LETTER_EF;
	a_cyr_lhrdcontxt[20] = CYRILLIC_SMALL_LETTER_HA;
	a_cyr_lhrdcontxt[21] = CYRILLIC_SMALL_LETTER_TSE;
	a_cyr_lhrdcontxt[22] = CYRILLIC_SMALL_LETTER_CHE;
	a_cyr_lhrdcontxt[23] = CYRILLIC_SMALL_LETTER_SHA;
	a_cyr_lhrdcontxt[24] = CYRILLIC_SMALL_LETTER_SHCHA;
	a_cyr_lhrdcontxt[25] = CYRILLIC_SMALL_LETTER_HARD_SIGN;
	a_cyr_lhrdcontxt[26] = CYRILLIC_SMALL_LETTER_YERU;
	a_cyr_lhrdcontxt[27] = CYRILLIC_SMALL_LETTER_E;
	a_cyr_lhrdcontxt[28] = CYRILLIC_SMALL_LETTER_YU;
	a_cyr_lhrdcontxt[29] = CYRILLIC_SMALL_LETTER_YA;
	a_cyr_lhrdcontxt[30] = SPACE;
	a_cyr_lhrdcontxt[31] = EXCLAMATION_MARK;
	a_cyr_lhrdcontxt[32] = QUOTATION_MARK;
	a_cyr_lhrdcontxt[33] = COMMA;
	a_cyr_lhrdcontxt[34] = FULL_STOP;


	a_cyr_uhrdcontxt[0]  = CYRILLIC_CAPITAL_LETTER_A;
	a_cyr_uhrdcontxt[1]  = CYRILLIC_CAPITAL_LETTER_BE;
	a_cyr_uhrdcontxt[2]  = CYRILLIC_CAPITAL_LETTER_VE;
	a_cyr_uhrdcontxt[3]  = CYRILLIC_CAPITAL_LETTER_GHE;
	a_cyr_uhrdcontxt[4]  = CYRILLIC_CAPITAL_LETTER_DE;
	a_cyr_uhrdcontxt[5]  = CYRILLIC_CAPITAL_LETTER_ZHE;
	a_cyr_uhrdcontxt[6]  = CYRILLIC_CAPITAL_LETTER_ZE;
	a_cyr_uhrdcontxt[7] = CYRILLIC_CAPITAL_LETTER_SHORT_I;
	a_cyr_uhrdcontxt[8] = CYRILLIC_CAPITAL_LETTER_KA;
	a_cyr_uhrdcontxt[9] = CYRILLIC_CAPITAL_LETTER_EL;
	a_cyr_uhrdcontxt[10] = CYRILLIC_CAPITAL_LETTER_EM;
	a_cyr_uhrdcontxt[11] = CYRILLIC_CAPITAL_LETTER_EN;
	a_cyr_uhrdcontxt[12] = CYRILLIC_CAPITAL_LETTER_O;
	a_cyr_uhrdcontxt[13] = CYRILLIC_CAPITAL_LETTER_PE;
	a_cyr_uhrdcontxt[14] = CYRILLIC_CAPITAL_LETTER_ER;
	a_cyr_uhrdcontxt[15] = CYRILLIC_CAPITAL_LETTER_ES;
	a_cyr_uhrdcontxt[16] = CYRILLIC_CAPITAL_LETTER_TE;
	a_cyr_uhrdcontxt[17] = CYRILLIC_CAPITAL_LETTER_U;
	a_cyr_uhrdcontxt[18] = CYRILLIC_CAPITAL_LETTER_SHORT_U;
	a_cyr_uhrdcontxt[19] = CYRILLIC_CAPITAL_LETTER_EF;
	a_cyr_uhrdcontxt[20] = CYRILLIC_CAPITAL_LETTER_HA;
	a_cyr_uhrdcontxt[21] = CYRILLIC_CAPITAL_LETTER_TSE;
	a_cyr_uhrdcontxt[22] = CYRILLIC_CAPITAL_LETTER_CHE;
	a_cyr_uhrdcontxt[23] = CYRILLIC_CAPITAL_LETTER_SHA;
	a_cyr_uhrdcontxt[24] = CYRILLIC_CAPITAL_LETTER_SHCHA;
	a_cyr_uhrdcontxt[25] = CYRILLIC_CAPITAL_LETTER_HARD_SIGN;
	a_cyr_uhrdcontxt[26] = CYRILLIC_CAPITAL_LETTER_YERU;
	a_cyr_uhrdcontxt[27] = CYRILLIC_CAPITAL_LETTER_E;
	a_cyr_uhrdcontxt[28] = CYRILLIC_CAPITAL_LETTER_YU;
	a_cyr_uhrdcontxt[29] = CYRILLIC_CAPITAL_LETTER_YA;
	a_cyr_lhrdcontxt[30] = 0;
	a_cyr_lhrdcontxt[31] = 0;
	a_cyr_lhrdcontxt[32] = 0;
	a_cyr_lhrdcontxt[33] = 0;
	a_cyr_lhrdcontxt[34] = 0;


	b_cyr_hrdcontxt	=false;
	blcase			= false;
	blvow			= false;  

}

void CConvert::C2T(wchar_t* wbuffer, int wbuflen)
{

	wstr = wbuffer;

	for(i=0;i<= (wstr.size());i++)
		{
			switch(wstr[i])
			{

/*************/
/* Uppercase */
/*************/

			case CYRILLIC_CAPITAL_LETTER_A:
				wstr[i] = LATIN_CAPITAL_LETTER_A;
				break;

			case CYRILLIC_CAPITAL_LETTER_BE:
				wstr[i] = LATIN_CAPITAL_LETTER_B;
				break;

			case CYRILLIC_CAPITAL_LETTER_VE:
				wstr[i] = LATIN_CAPITAL_LETTER_V;
				break;

			case CYRILLIC_CAPITAL_LETTER_GHE:
				wstr[i] = LATIN_CAPITAL_LETTER_H;
				break;

			case CYRILLIC_CAPITAL_LETTER_DE:
				wstr[i] = LATIN_CAPITAL_LETTER_D;
				break;

			case CYRILLIC_CAPITAL_LETTER_IE:
// Followed by a_cyr_lcase or a_cyr_ucase
					for (x = 0; x <= 34; x ++)
					{
						if (wstr[i+1] == a_cyr_lcase[x])
						{
							blcase = true;
							break;
						}						
					}
// If in the beginning of a word

				if		((wstr[i-1]==SPACE) || (wstr[i-1]==CARRIAGE_RETURN) || (wstr[i-1]==0)) 
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_E;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_E;
					// gcc workaround
					wstr.insert(i, L"J");
					i++;
				}
				else if (wstr[i-1] != LATIN_CAPITAL_LETTER_L) 
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_E;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_E;

					wstr.insert(i, L"I");
					i++;
				}
				else
				{
					wstr[i] = LATIN_CAPITAL_LETTER_E;
				}
				blcase = false;
				break;

			case CYRILLIC_CAPITAL_LETTER_IO:
// Followed by a_cyr_lcase or a_cyr_ucase
					for (x = 0; x <= 34; x ++)
					{
						if (wstr[i+1] == a_cyr_lcase[x])
						{
							blcase = true;
							break;
						}						
					}
// If in the beginning of a word

				if ((wstr[i-1]==SPACE) || (wstr[i-1]==CARRIAGE_RETURN) || (wstr[i-1]==0)) 
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_O;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_O;

					wstr.insert(i, L"J");
					i++;
				}
				// Before LATIN_CAPITAL_LETTER_L ?
				else if (wstr[i-1] == LATIN_CAPITAL_LETTER_L) 
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_O;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_O;
				}
				// Otherwise
				else
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_O;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_O;

					wstr.insert(i, L"I");
					i++;
				}
				blcase = false;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_ZHE:
				wstr[i] = LATIN_CAPITAL_LETTER_Z;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_ZE:
				wstr[i] = LATIN_CAPITAL_LETTER_Z;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_BYELORUSSIAN_UKRAINIAN_I:
				wstr[i] = LATIN_CAPITAL_LETTER_I;
				break;

			case CYRILLIC_CAPITAL_LETTER_I:
				wstr[i] = LATIN_CAPITAL_LETTER_I;
				break;
				
			case CYRILLIC_CAPITAL_LETTER_SHORT_I:
				wstr[i] = LATIN_CAPITAL_LETTER_J;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_KA:
				wstr[i] = LATIN_CAPITAL_LETTER_K;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_EL:
				wstr[i] = LATIN_CAPITAL_LETTER_L;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_EM:
				wstr[i] = LATIN_CAPITAL_LETTER_M;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_EN:
				wstr[i] = LATIN_CAPITAL_LETTER_N;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_O:
				wstr[i] = LATIN_CAPITAL_LETTER_O;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_PE:
				wstr[i] = LATIN_CAPITAL_LETTER_P;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_ER:
				wstr[i] = LATIN_CAPITAL_LETTER_R;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_ES:
				wstr[i] = LATIN_CAPITAL_LETTER_S;
				break;
			
			
			case CYRILLIC_CAPITAL_LETTER_TE:
				wstr[i] = LATIN_CAPITAL_LETTER_T;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_U:
				wstr[i] = LATIN_CAPITAL_LETTER_U;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_SHORT_U:
				wstr[i] = LATIN_CAPITAL_LETTER_U;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_EF:
				wstr[i] = LATIN_CAPITAL_LETTER_F;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_HA:
// Followed by a_cyr_lcase or a_cyr_ucase
				for (x = 0; x <= 34; x ++)
				{
					if (wstr[i+1] == a_cyr_lcase[x])
					{
						blcase = true;
						break;
					}						
				}
				if (blcase == true )
				{
					wstr[i] = LATIN_SMALL_LETTER_H;
					wstr.insert(i,L"C");
					i++;
					
				}
				else
				{
					wstr[i] = LATIN_CAPITAL_LETTER_H;
					wstr.insert(i,L"C");	
					i++;
				}
				blcase = false;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_TSE:
				wstr[i] = LATIN_CAPITAL_LETTER_C;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_CHE:
				wstr[i] = LATIN_CAPITAL_LETTER_C;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_SHA:
				wstr[i] = LATIN_CAPITAL_LETTER_S;
				break;

			case CYRILLIC_CAPITAL_LETTER_SHCHA:
				wstr[i] = LATIN_CAPITAL_LETTER_C;
				wstr.insert(i, L"S");
				i++;
				break;

			
			case CYRILLIC_CAPITAL_LETTER_YERU:
				wstr[i] = LATIN_CAPITAL_LETTER_Y;
				break;

			case CYRILLIC_CAPITAL_LETTER_SOFT_SIGN:
				wstr[i] = APOSTROPHE;
				break;

			case CYRILLIC_CAPITAL_LETTER_E:
				wstr[i] = LATIN_CAPITAL_LETTER_E;
				break;

			case CYRILLIC_CAPITAL_LETTER_YU:
				// Followed by a_cyr_lcase or a_cyr_ucase
				for (x = 0; x <= 34; x ++)
				{
					if (wstr[i+1] == a_cyr_lcase[x])
					{
						blcase = true;
						break;
					}						
				}

				// If in the beginning of a word
				if		((wstr[i-1]==SPACE) || (wstr[i-1]==CARRIAGE_RETURN) || (wstr[i-1]==0)) 
				{
					if (blcase == true ) 
					{wstr[i] = LATIN_SMALL_LETTER_U;}
					else
					{wstr[i] = LATIN_CAPITAL_LETTER_U;}
					wstr.insert(i, L"J");
					i++;
				}
				// Before LATIN_CAPITAL_LETTER_L ?
				else if (wstr[i-1] == LATIN_CAPITAL_LETTER_L) 
				{
					if (blcase == true ) 
					{wstr[i] = LATIN_SMALL_LETTER_U;}
					else
					{wstr[i] = LATIN_CAPITAL_LETTER_U;}
				}
				// Otherwise
				else
				{
					if (blcase == true ) 
					{wstr[i] = LATIN_SMALL_LETTER_U;}
					else
					{wstr[i] = LATIN_CAPITAL_LETTER_U;}
					wstr.insert(i, L"I");
					i++;
				}
				blcase = false;
				break;

			case CYRILLIC_CAPITAL_LETTER_YA:
			// Followed by a_cyr_lcase or a_cyr_ucase
				for (x = 0; x <= 34; x ++)
				{
					if (wstr[i+1] == a_cyr_lcase[x])
					{
						blcase = true;
						break;
					}
					if (wstr[i+1] == SPACE) 
						blcase = true;
				}
				// If in the beginning of a word
				if ((wstr[i-1]==SPACE) || (wstr[i-1]==CARRIAGE_RETURN) || (wstr[i-1]==0)) 
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_A;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_A;

					wstr.insert(i, L"J");
					i++;
				}
				// Before LATIN_CAPITAL_LETTER_L ?
				else if (wstr[i-1] == LATIN_CAPITAL_LETTER_L) 
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_A;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_A;
				}
				// Otherwise
				else
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_A;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_A;

					wstr.insert(i, L"I");
					i++;
				}
				blcase = false;
				break;

/*************/
/* Lowercase */
/*************/

			case CYRILLIC_SMALL_LETTER_A:
				wstr[i] = LATIN_SMALL_LETTER_A;
				break;

			case CYRILLIC_SMALL_LETTER_BE:
				wstr[i] = LATIN_SMALL_LETTER_B;
				break;

			case CYRILLIC_SMALL_LETTER_VE:
				wstr[i] = LATIN_SMALL_LETTER_V;
				break;

			case CYRILLIC_SMALL_LETTER_GHE:
				wstr[i] = LATIN_SMALL_LETTER_H;
				break;

			case CYRILLIC_SMALL_LETTER_DE:
				wstr[i] = LATIN_SMALL_LETTER_D;
				break;

			case CYRILLIC_SMALL_LETTER_IE:
// if after another vowel
				for (x = 0; x <= 5; x ++)
				{
					if (wstr[i-1] ==  a_lat_lvow[x])
					{
						blvow = true;
						break;
					}						
				}
//				
				if		((wstr[i-1] == SPACE) || (wstr[i-1] == CARRIAGE_RETURN) || (wstr[i-1] == 0)) 
				{
					wstr[i] = LATIN_SMALL_LETTER_E;
					wstr.insert(i, L"j");
					i++;
				}
				else if (blvow == true)
				{
					wstr[i] = LATIN_SMALL_LETTER_E;
					wstr.insert(i, L"j");
					i++;
				}
				else if (wstr[i-1] != LATIN_SMALL_LETTER_L) 
				{
					wstr[i] = LATIN_SMALL_LETTER_E;
					wstr.insert(i, L"i");
					i++;
				}
				else
				{
					wstr[i] = LATIN_SMALL_LETTER_E;
				}
				blvow = false;
				break;

			case CYRILLIC_SMALL_LETTER_IO:
// if after another vowel
				// is in the beginning of a word? 				
				if ((wstr[i-1] == SPACE) || (wstr[i-1] == CARRIAGE_RETURN) || (wstr[i-1] == 0)) 
				{
					wstr[i] = LATIN_SMALL_LETTER_O;
					wstr.insert(i, L"j");
					i++;
				}
				// preceeded by a vowel?
				else if 
				((wstr[i-1]==LATIN_SMALL_LETTER_A) || (wstr[i-1]==LATIN_SMALL_LETTER_E) || (wstr[i-1]==LATIN_SMALL_LETTER_O) || 
				( wstr[i-1]==LATIN_SMALL_LETTER_I) || (wstr[i-1]==LATIN_SMALL_LETTER_U) || (wstr[i-1]==LATIN_SMALL_LETTER_U) || 
				(wstr[i-1]==0))
				{
					{wstr[i] = LATIN_SMALL_LETTER_O;}
					wstr.insert(i, L"j");
					i++;
				}
				// Before LATIN_SMALL_LETTER_L ?
				else if (wstr[i-1] == LATIN_SMALL_LETTER_L) 
				{
						wstr[i] = LATIN_SMALL_LETTER_O;
				}
				// Otherwise
				else
				{
					wstr[i] = LATIN_SMALL_LETTER_O;
					wstr.insert(i, L"i");
					i++;
				}
				break;

				break;
			case CYRILLIC_SMALL_LETTER_ZHE:
				wstr[i] = LATIN_SMALL_LETTER_Z;
				break;
			
			case CYRILLIC_SMALL_LETTER_ZE:
				wstr[i] = LATIN_SMALL_LETTER_Z;
				break;

			case CYRILLIC_SMALL_LETTER_BYELORUSSIAN_UKRAINIAN_I:
				wstr[i] = LATIN_SMALL_LETTER_I;
				break;
			
			case CYRILLIC_SMALL_LETTER_I:
				wstr[i] = LATIN_SMALL_LETTER_I;
				break;

			case CYRILLIC_SMALL_LETTER_SHORT_I:
				wstr[i] = LATIN_SMALL_LETTER_J;
				break;
			
			case CYRILLIC_SMALL_LETTER_KA:
				wstr[i] = LATIN_SMALL_LETTER_K;
				break;
			
			case CYRILLIC_SMALL_LETTER_EL:
				wstr[i] = LATIN_SMALL_LETTER_L;
				break;
			
			case CYRILLIC_SMALL_LETTER_EM:
				wstr[i] = LATIN_SMALL_LETTER_M;
				break;
			
			case CYRILLIC_SMALL_LETTER_EN:
				wstr[i] = LATIN_SMALL_LETTER_N;
				break;
			
			case CYRILLIC_SMALL_LETTER_O:
				wstr[i] = LATIN_SMALL_LETTER_O;
				break;
			
			case CYRILLIC_SMALL_LETTER_PE:
				wstr[i] = LATIN_SMALL_LETTER_P;
				break;
			
			case CYRILLIC_SMALL_LETTER_ER:
				wstr[i] = LATIN_SMALL_LETTER_R;
				break;
			
			case CYRILLIC_SMALL_LETTER_ES:
				wstr[i] = LATIN_SMALL_LETTER_S;
				break;
			
			
			case CYRILLIC_SMALL_LETTER_TE:
				wstr[i] = LATIN_SMALL_LETTER_T;
				break;
			
			case CYRILLIC_SMALL_LETTER_U:
				wstr[i] = LATIN_SMALL_LETTER_U;
				break;
			
			case CYRILLIC_SMALL_LETTER_SHORT_U:
				wstr[i] = LATIN_SMALL_LETTER_U;
				break;
			
			case CYRILLIC_SMALL_LETTER_EF:
				wstr[i] = LATIN_SMALL_LETTER_F;
				break;
			
			case CYRILLIC_SMALL_LETTER_HA:
				wstr[i] = LATIN_SMALL_LETTER_H;
				wstr.insert(i,L"c");
				i++;
				break;
			
			case CYRILLIC_SMALL_LETTER_TSE:
				wstr[i] = LATIN_SMALL_LETTER_C;
				break;
			
			case CYRILLIC_SMALL_LETTER_CHE:
				wstr[i] = LATIN_SMALL_LETTER_C;
				break;
			
			case CYRILLIC_SMALL_LETTER_SHA:
				wstr[i] = LATIN_SMALL_LETTER_S;
				break;

			case CYRILLIC_SMALL_LETTER_SHCHA:
				wstr[i] = LATIN_SMALL_LETTER_C;
				wstr.insert(i, L"s");
				i++;
				break;

			
			case CYRILLIC_SMALL_LETTER_YERU:
				wstr[i] = LATIN_SMALL_LETTER_Y;
				break;

			case CYRILLIC_SMALL_LETTER_SOFT_SIGN:
				wstr[i] = APOSTROPHE;
				break;

			case CYRILLIC_SMALL_LETTER_E:
				wstr[i] = LATIN_SMALL_LETTER_E;
				break;

			case CYRILLIC_SMALL_LETTER_YU:
				// is in the beginning of a word? 				
				if ((wstr[i-1] == SPACE) || (wstr[i-1] == CARRIAGE_RETURN) || (wstr[i-1] == 0)) 
				{
					wstr[i] = LATIN_SMALL_LETTER_U;
					wstr.insert(i, L"j");
					i++;
				}
				// preceeded by a vowel?
				else if 
				((wstr[i-1]==LATIN_SMALL_LETTER_A) || (wstr[i-1]==LATIN_SMALL_LETTER_E) || (wstr[i-1]==LATIN_SMALL_LETTER_O) || 
				( wstr[i-1]==LATIN_SMALL_LETTER_I) || (wstr[i-1]==LATIN_SMALL_LETTER_U) || (wstr[i-1]==LATIN_SMALL_LETTER_Y) || 
				(wstr[i-1]==0))
				{
					{wstr[i] = LATIN_SMALL_LETTER_U;}
					wstr.insert(i, L"j");
					i++;
				}
				// Before LATIN_SMALL_LETTER_L ?
				else if (wstr[i-1] == LATIN_SMALL_LETTER_L) 
				{
						wstr[i] = LATIN_SMALL_LETTER_U;
				}
				// Otherwise
				else
				{
					wstr[i] = LATIN_SMALL_LETTER_U;
					wstr.insert(i, L"i");
					i++;
				}
				break;

			case CYRILLIC_SMALL_LETTER_YA:
				// is in the beginning of a word? 				
				if ((wstr[i-1] == SPACE) || (wstr[i-1] == CARRIAGE_RETURN) || (wstr[i-1] == 0)) 
				{
					wstr[i] = LATIN_SMALL_LETTER_A;
					wstr.insert(i, L"j");
					i++;
				}
				// preceeded by a vowel?
				else if 
				((wstr[i-1]==LATIN_SMALL_LETTER_A) || (wstr[i-1]==LATIN_SMALL_LETTER_E) || (wstr[i-1]==LATIN_SMALL_LETTER_O) || 
				( wstr[i-1]==LATIN_SMALL_LETTER_I) || (wstr[i-1]==LATIN_SMALL_LETTER_U) || (wstr[i-1]==LATIN_SMALL_LETTER_Y) || 
				(wstr[i-1]==0))
				{
					{wstr[i] = LATIN_SMALL_LETTER_A;}
					wstr.insert(i, L"j");
					i++;
				}
				// Before LATIN_SMALL_LETTER_L ?
				else if (wstr[i-1] == LATIN_SMALL_LETTER_L) 
				{
						wstr[i] = LATIN_SMALL_LETTER_A;
				}
				// Otherwise
				else
				{
					wstr[i] = LATIN_SMALL_LETTER_A;
					wstr.insert(i, L"i");
					i++;
				}
				break;


			}
	}
	if (wbuflen >= wcslen(wstr.c_str()))
	{
		//scpy(wbuffer,wstr.c_str());
		//pwstring = wstr.c_str();
		//while(((*wbuffer++)=(*pwstring++))!=(L'\0'));
		wDestination = wbuffer;
		wSource = wstr.c_str();
		while((*wDestination++ = *wSource++) != 0);

	// insert exception
	}
}

void CConvert::C2L(wchar_t* wbuffer, int wbuflen)
{

	wstr = wbuffer;

	for(i=0;i<= (wstr.size());i++)
	{
			switch(wstr[i])
			{

/*************/
/* Uppercase */
/*************/

			case CYRILLIC_CAPITAL_LETTER_A:
				wstr[i] = LATIN_CAPITAL_LETTER_A;
				break;

			case CYRILLIC_CAPITAL_LETTER_BE:
				wstr[i] = LATIN_CAPITAL_LETTER_B;
				break;

			case CYRILLIC_CAPITAL_LETTER_VE:
				wstr[i] = LATIN_CAPITAL_LETTER_V;
				break;

			case CYRILLIC_CAPITAL_LETTER_GHE:
				wstr[i] = LATIN_CAPITAL_LETTER_H;
				break;

			case CYRILLIC_CAPITAL_LETTER_DE:
				wstr[i] = LATIN_CAPITAL_LETTER_D;
				break;

			case CYRILLIC_CAPITAL_LETTER_IE:
// Followed by a_cyr_lcase or a_cyr_ucase
				for (x = 0; x <= 34; x ++)
				{
					if (wstr[i+1] == a_cyr_lcase[x])
					{
						blcase = true;
						break;
					}						
				}
// If in the beginning of a word

				if		((wstr[i-1]==SPACE) || (wstr[i-1]==CARRIAGE_RETURN) || (wstr[i-1]==0)) 
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_E;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_E;

					wstr.insert(i, L"J");
					i++;
				}
				else if (wstr[i-1] != LATIN_CAPITAL_LETTER_L) 
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_E;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_E;

					wstr.insert(i, L"I");
					i++;
				}
				else
				{
					wstr[i] = LATIN_CAPITAL_LETTER_E;
				}
				blcase = false;
				break;

			case CYRILLIC_CAPITAL_LETTER_IO:
			// Weither followed by a_cyr_lcase or a_cyr_ucase 
				for (x = 0; x <= 34; x ++)
				{
					if (wstr[i+1] == a_cyr_lcase[x])
					{
						blcase = true;
						break;
					}
					if (wstr[i+1] == SPACE) 
						blcase = true;
				}

				// is in the beginning of a word? 
				if ((wstr[i-1]==SPACE) || (wstr[i-1]==CARRIAGE_RETURN) || (wstr[i-1]==0))
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_O;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_O;

					wstr.insert(i, L"J");
					i++;
				}

				// preceeded by a vowel?
				else if 
				((wstr[i-1]==LATIN_CAPITAL_LETTER_A) || (wstr[i-1]==LATIN_CAPITAL_LETTER_E) || (wstr[i-1]==LATIN_CAPITAL_LETTER_O) || 
				( wstr[i-1]==LATIN_CAPITAL_LETTER_I) || (wstr[i-1]==LATIN_CAPITAL_LETTER_U) || (wstr[i-1]==LATIN_CAPITAL_LETTER_Y) || 
				(wstr[i-1]==0))
				{
					if (blcase == true ) 
					{wstr[i] = LATIN_SMALL_LETTER_O;}
					else
					{wstr[i] = LATIN_CAPITAL_LETTER_O;}
					wstr.insert(i, L"J");
					i++;
				}

				// Before 'L' ?
				else if (wstr[i-1] == LATIN_CAPITAL_LETTER_L) 
				{
					if (blcase == true ) 
					{wstr[i] = LATIN_SMALL_LETTER_O;}
					else
					{wstr[i] = LATIN_CAPITAL_LETTER_O;}
				}
				// Otherwise
				else
				{
					if (blcase == true ) 
					{wstr[i] = LATIN_SMALL_LETTER_O;}
					else
					{wstr[i] = LATIN_CAPITAL_LETTER_O;}
					wstr.insert(i, L"I");
					i++;
				}

				blcase = false;
				break;

			case CYRILLIC_CAPITAL_LETTER_ZHE:
				wstr[i] = LATIN_CAPITAL_LETTER_Z_WITH_CARON;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_ZE:
				if ((wstr[i+1]==CYRILLIC_SMALL_LETTER_SOFT_SIGN) || (wstr[i+1]==CYRILLIC_CAPITAL_LETTER_SOFT_SIGN)) 
				{
					wstr[i] = LATIN_CAPITAL_LETTER_Z_WITH_ACUTE;
					wstr.erase((i+1),1);
					break;
				}
				else 
				{
					wstr[i] = LATIN_CAPITAL_LETTER_Z;
					break;
				}
				
			case CYRILLIC_CAPITAL_LETTER_BYELORUSSIAN_UKRAINIAN_I:
				wstr[i] = LATIN_CAPITAL_LETTER_I;
				break;

			case CYRILLIC_CAPITAL_LETTER_I:
				wstr[i] = LATIN_SMALL_LETTER_I;
				break;
				
			case CYRILLIC_CAPITAL_LETTER_SHORT_I:
				wstr[i] = LATIN_CAPITAL_LETTER_J;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_KA:
				wstr[i] = LATIN_CAPITAL_LETTER_K;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_EL:
// Check weither the following letter is hard or soft
				for (x = 0; x <= 35; x ++)
				{
					if ((wstr[i+1] == a_cyr_lhrdcontxt[x]) || (wstr[i+1] == a_cyr_uhrdcontxt[x]))
					{
						b_cyr_hrdcontxt = true;
						break;
					}						
				}
				if (b_cyr_hrdcontxt == true)
				{
					wstr[i] = LATIN_CAPITAL_LETTER_L_WITH_STROKE;
				}
				else if ((wstr[i+1]==CYRILLIC_SMALL_LETTER_SOFT_SIGN) || (wstr[i+1]==CYRILLIC_CAPITAL_LETTER_SOFT_SIGN))
				{
					wstr[i] = LATIN_CAPITAL_LETTER_L;
					wstr.erase((i+1),1);
				}
				else 
				{
					wstr[i] = LATIN_CAPITAL_LETTER_L;
				}
				b_cyr_hrdcontxt = false;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_EM:
				wstr[i] = LATIN_CAPITAL_LETTER_M;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_EN:
				if ((wstr[i+1]==CYRILLIC_SMALL_LETTER_SOFT_SIGN) || (wstr[i+1]==CYRILLIC_CAPITAL_LETTER_SOFT_SIGN)) 
				{
					wstr[i] = LATIN_CAPITAL_LETTER_N_WITH_ACUTE;
					wstr.erase((i+1),1);
					break;
				}
				else 
				{
					wstr[i] = LATIN_CAPITAL_LETTER_N;
					break;
				}
			
			case CYRILLIC_CAPITAL_LETTER_O:
				wstr[i] = LATIN_CAPITAL_LETTER_O;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_PE:
				wstr[i] = LATIN_CAPITAL_LETTER_P;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_ER:
				wstr[i] = LATIN_CAPITAL_LETTER_R;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_ES:
				if ((wstr[i+1]==CYRILLIC_SMALL_LETTER_SOFT_SIGN) || (wstr[i+1]==CYRILLIC_CAPITAL_LETTER_SOFT_SIGN)) 
				{
					wstr[i] = LATIN_CAPITAL_LETTER_S_WITH_ACUTE;
					wstr.erase((i+1),1);
					break;
				}
				else 
				{
					wstr[i] = LATIN_CAPITAL_LETTER_S;
					break;
				}
			
			
			case CYRILLIC_CAPITAL_LETTER_TE:
				wstr[i] = LATIN_CAPITAL_LETTER_T;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_U:
				wstr[i] = LATIN_CAPITAL_LETTER_U;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_SHORT_U:
				wstr[i] = LATIN_CAPITAL_LETTER_U_WITH_BREVE;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_EF:
				wstr[i] = LATIN_CAPITAL_LETTER_F;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_HA:
// Followed by a_cyr_lcase or a_cyr_ucase
				for (x = 0; x <= 34; x ++)
				{
					if (wstr[i+1] == a_cyr_lcase[x])
					{
						blcase = true;
						break;
					}
				}
				if (blcase == true )
				{
					wstr[i] = LATIN_SMALL_LETTER_H;
					wstr.insert(i,L"C");
					i++;
					
				}
				else
				{
					wstr[i] = LATIN_CAPITAL_LETTER_H;
					wstr.insert(i,L"C");	
					i++;
				}
				blcase = false;
				break;

			
			case CYRILLIC_CAPITAL_LETTER_TSE:
				if ((wstr[i+1]==CYRILLIC_SMALL_LETTER_SOFT_SIGN) || (wstr[i+1]==CYRILLIC_CAPITAL_LETTER_SOFT_SIGN)) 
				{
					wstr[i] = LATIN_CAPITAL_LETTER_C_WITH_ACUTE;
					wstr.erase((i+1),1);
					break;
				}
				else 
				{
					wstr[i] = LATIN_CAPITAL_LETTER_C;
					break;
				}
			
			case CYRILLIC_CAPITAL_LETTER_CHE:
				wstr[i] = LATIN_CAPITAL_LETTER_C_WITH_CARON;
				break;
			
			case CYRILLIC_CAPITAL_LETTER_SHA:
				wstr[i] = LATIN_CAPITAL_LETTER_S_WITH_CARON;
				break;

			case CYRILLIC_CAPITAL_LETTER_SHCHA:
				wstr[i] = LATIN_CAPITAL_LETTER_C_WITH_CARON;
				wstr.insert(i, L"\x0160");
				i++;
				break;

			
			case CYRILLIC_CAPITAL_LETTER_YERU:
				wstr[i] = LATIN_CAPITAL_LETTER_Y;
				break;

			case CYRILLIC_CAPITAL_LETTER_SOFT_SIGN:
				wstr[i] = APOSTROPHE;
				break;

			case CYRILLIC_CAPITAL_LETTER_E:
				wstr[i] = LATIN_CAPITAL_LETTER_E;
				break;

			case CYRILLIC_CAPITAL_LETTER_YU:
			// Weither followed by a_cyr_lcase or a_cyr_ucase 
				for (x = 0; x <= 34; x ++)
				{
					if (wstr[i+1] == a_cyr_lcase[x])
					{
						blcase = true;
						break;
					}
					if (wstr[i+1] == SPACE) 
						blcase = true;
				}

				// is in the beginning of a word? 
				if ((wstr[i-1]==SPACE) || (wstr[i-1]==CARRIAGE_RETURN) || (wstr[i-1]==0))
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_U;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_U;

					wstr.insert(i, L"J");
					i++;
				}

				// preceeded by a vowel?
				else if 
				((wstr[i-1]==LATIN_CAPITAL_LETTER_A) || (wstr[i-1]==LATIN_CAPITAL_LETTER_E) || (wstr[i-1]==LATIN_CAPITAL_LETTER_O) || 
				( wstr[i-1]==LATIN_CAPITAL_LETTER_I) || (wstr[i-1]==LATIN_CAPITAL_LETTER_U) || (wstr[i-1]==LATIN_CAPITAL_LETTER_Y) || 
				(wstr[i-1]==0))
				{
					if (blcase == true ) 
					{wstr[i] = LATIN_SMALL_LETTER_U;}
					else
					{wstr[i] = LATIN_CAPITAL_LETTER_U;}
					wstr.insert(i, L"J");
					i++;
				}

				// Before 'L' ?
				else if (wstr[i-1] == LATIN_CAPITAL_LETTER_L) 
				{
					if (blcase == true ) 
					{wstr[i] = LATIN_SMALL_LETTER_U;}
					else
					{wstr[i] = LATIN_CAPITAL_LETTER_U;}
				}
				// Otherwise
				else
				{
					if (blcase == true ) 
					{wstr[i] = LATIN_SMALL_LETTER_U;}
					else
					{wstr[i] = LATIN_CAPITAL_LETTER_U;}
					wstr.insert(i, L"I");
					i++;
				}
				blcase = false;
				break;

			case CYRILLIC_CAPITAL_LETTER_YA:
			// Weither followed by a_cyr_lcase or a_cyr_ucase 
				for (x = 0; x <= 34; x ++)
				{
					if (wstr[i+1] == a_cyr_lcase[x])
					{
						blcase = true;
						break;
					}
					if (wstr[i+1] == SPACE) 
						blcase = true;
				}

				// is in the beginning of a word? 
				if ((wstr[i-1]==SPACE) || (wstr[i-1]==CARRIAGE_RETURN) || (wstr[i-1]==0))
				{
					if (blcase == true ) 
						wstr[i] = LATIN_SMALL_LETTER_A;
					else
						wstr[i] = LATIN_CAPITAL_LETTER_A;

					wstr.insert(i, L"J");
					i++;
				}

				// preceeded by a vowel?
				else if 
				((wstr[i-1]==LATIN_CAPITAL_LETTER_A) || (wstr[i-1]==LATIN_CAPITAL_LETTER_E) || (wstr[i-1]==LATIN_CAPITAL_LETTER_O) || 
				( wstr[i-1]==LATIN_CAPITAL_LETTER_I) || (wstr[i-1]==LATIN_CAPITAL_LETTER_U) || (wstr[i-1]==LATIN_CAPITAL_LETTER_Y) || 
				(wstr[i-1]==0))
				{
					if (blcase == true ) 
					{wstr[i] = LATIN_SMALL_LETTER_A;}
					else
					{wstr[i] = LATIN_CAPITAL_LETTER_A;}
					wstr.insert(i, L"J");
					i++;
				}

				// Before 'L' ?
				else if (wstr[i-1] == LATIN_CAPITAL_LETTER_L) 
				{
					if (blcase == true ) 
					{wstr[i] = LATIN_SMALL_LETTER_A;}
					else
					{wstr[i] = LATIN_CAPITAL_LETTER_A;}
				}
				// Otherwise
				else
				{
					if (blcase == true ) 
					{wstr[i] = LATIN_SMALL_LETTER_A;}
					else
					{wstr[i] = LATIN_CAPITAL_LETTER_A;}
					wstr.insert(i, L"I");
					i++;
				}

				blcase = false;
				break;

/*************/
/* Lowercase */
/*************/

			case CYRILLIC_SMALL_LETTER_A:
				wstr[i] = LATIN_SMALL_LETTER_A;
				break;

			case CYRILLIC_SMALL_LETTER_BE:
				wstr[i] = LATIN_SMALL_LETTER_B;
				break;

			case CYRILLIC_SMALL_LETTER_VE:
				wstr[i] = LATIN_SMALL_LETTER_V;
				break;

			case CYRILLIC_SMALL_LETTER_GHE:
				wstr[i] = LATIN_SMALL_LETTER_H;
				break;

			case CYRILLIC_SMALL_LETTER_DE:
				wstr[i] = LATIN_SMALL_LETTER_D;
				break;

			case CYRILLIC_SMALL_LETTER_IE:
// if after another vowel
				for (x = 0; x <= 5; x ++)
				{
					if (wstr[i-1] ==  a_lat_lvow[x])
					{
						blvow = true;
						break;
					}						
				}
//				
				if ((wstr[i-1] == SPACE) || (wstr[i-1] == CARRIAGE_RETURN) || (wstr[i-1] == 0)) 
				{
					wstr[i] = LATIN_SMALL_LETTER_E;
					wstr.insert(i, L"j");
					i++;
				}
				else if (blvow == true)
				{
					wstr[i] = LATIN_SMALL_LETTER_E;
					wstr.insert(i, L"j");
					i++;
				}
				else if (wstr[i-1] != LATIN_SMALL_LETTER_L) 
				{
					wstr[i] = LATIN_SMALL_LETTER_E;
					wstr.insert(i, L"i");
					i++;
				}
				else
				{
					wstr[i] = LATIN_SMALL_LETTER_E;
				}
				blvow = false;
				break;

			case CYRILLIC_SMALL_LETTER_IO:
				// is in the beginning of a word? 
				if ((wstr[i-1] == SPACE) || (wstr[i-1] == CARRIAGE_RETURN) || (wstr[i-1] == 0)) 
				{
					wstr[i] = LATIN_SMALL_LETTER_O;
					wstr.insert(i, L"j");
					i++;
				}

				// preceeded by a vowel?
				else if 
				((wstr[i-1]==LATIN_SMALL_LETTER_A) || (wstr[i-1]==LATIN_SMALL_LETTER_E) || (wstr[i-1]==LATIN_SMALL_LETTER_O) || 
				( wstr[i-1]==LATIN_SMALL_LETTER_I) || (wstr[i-1]==LATIN_SMALL_LETTER_U) || (wstr[i-1]==LATIN_SMALL_LETTER_Y) || 
				(wstr[i-1]==0))
				{
					{wstr[i] = LATIN_SMALL_LETTER_O;}
					wstr.insert(i, L"j");
					i++;
				}
				// Before 'l' ?
				else if (wstr[i-1] == LATIN_SMALL_LETTER_L) 
				{
						wstr[i] = LATIN_SMALL_LETTER_O;
				}
				// Otherwise
				else
				{
					wstr[i] = LATIN_SMALL_LETTER_O;
					wstr.insert(i, L"i");
					i++;
				}
				break;


				break;
			case CYRILLIC_SMALL_LETTER_ZHE:
				wstr[i] = LATIN_SMALL_LETTER_Z_WITH_CARON;
				break;
			
			case CYRILLIC_SMALL_LETTER_ZE:
				if ((wstr[i+1]==CYRILLIC_SMALL_LETTER_SOFT_SIGN) || (wstr[i+1]==CYRILLIC_CAPITAL_LETTER_SOFT_SIGN)) 
				{
					wstr[i] = LATIN_SMALL_LETTER_Z_WITH_ACUTE;
					wstr.erase((i+1),1);
					break;
				}
				else 
				{
					wstr[i] = LATIN_SMALL_LETTER_Z;
					break;
				}

			case CYRILLIC_SMALL_LETTER_BYELORUSSIAN_UKRAINIAN_I:
				wstr[i] = LATIN_SMALL_LETTER_I;
				break;
			
			case CYRILLIC_SMALL_LETTER_I:
				wstr[i] = LATIN_SMALL_LETTER_I;
				break;

			case CYRILLIC_SMALL_LETTER_SHORT_I:
				wstr[i] = LATIN_SMALL_LETTER_J;
				break;
			
			case CYRILLIC_SMALL_LETTER_KA:
				wstr[i] = LATIN_SMALL_LETTER_K;
				break;
			
			case CYRILLIC_SMALL_LETTER_EL:
// Check weither the following letter is hard or soft
				for (x = 0; x <= 35; x ++)
				{
					if (wstr[i+1] == a_cyr_lhrdcontxt[x])
					{
						b_cyr_hrdcontxt = true;
						break;
					}						
				}
				if (b_cyr_hrdcontxt == true)
				{
					wstr[i] = LATIN_SMALL_LETTER_L_WITH_STROKE;
				}

				else if ((wstr[i+1]==CYRILLIC_SMALL_LETTER_SOFT_SIGN) || (wstr[i+1]==CYRILLIC_CAPITAL_LETTER_SOFT_SIGN))
				{
					wstr[i] = LATIN_SMALL_LETTER_L;
					wstr.erase((i+1),1);
				}
				else 
				{
					wstr[i] = LATIN_SMALL_LETTER_L;
				}
				b_cyr_hrdcontxt = false;
				break;
			
			case CYRILLIC_SMALL_LETTER_EM:
				wstr[i] = LATIN_SMALL_LETTER_M;
				break;
			
			case CYRILLIC_SMALL_LETTER_EN:
				if ((wstr[i+1]==CYRILLIC_SMALL_LETTER_SOFT_SIGN) || (wstr[i+1]==CYRILLIC_CAPITAL_LETTER_SOFT_SIGN)) 
				{
					wstr[i] = LATIN_SMALL_LETTER_N_WITH_ACUTE;
					wstr.erase((i+1),1);
					break;
				}
				else 
				{
					wstr[i] = LATIN_SMALL_LETTER_N;
					break;
				}
			
			case CYRILLIC_SMALL_LETTER_O:
				wstr[i] = LATIN_SMALL_LETTER_O;
				break;
			
			case CYRILLIC_SMALL_LETTER_PE:
				wstr[i] = LATIN_SMALL_LETTER_P;
				break;
			
			case CYRILLIC_SMALL_LETTER_ER:
				wstr[i] = LATIN_SMALL_LETTER_R;
				break;
			
			case CYRILLIC_SMALL_LETTER_ES:
				if ((wstr[i+1]==CYRILLIC_SMALL_LETTER_SOFT_SIGN) || (wstr[i+1]==CYRILLIC_CAPITAL_LETTER_SOFT_SIGN)) 
				{
					wstr[i] = LATIN_SMALL_LETTER_S_WITH_ACUTE;
					wstr.erase((i+1),1);
					break;
				}
				else 
				{
					wstr[i] = LATIN_SMALL_LETTER_S;
					break;
				}
			
			
			case CYRILLIC_SMALL_LETTER_TE:
				wstr[i] = LATIN_SMALL_LETTER_T;
				break;
			
			case CYRILLIC_SMALL_LETTER_U:
				wstr[i] = LATIN_SMALL_LETTER_U;
				break;
			
			case CYRILLIC_SMALL_LETTER_SHORT_U:
				wstr[i] = LATIN_SMALL_LETTER_U_WITH_BREVE;
				break;
			
			case CYRILLIC_SMALL_LETTER_EF:
				wstr[i] = LATIN_SMALL_LETTER_F;
				break;
			
			case CYRILLIC_SMALL_LETTER_HA:
				wstr[i] = LATIN_SMALL_LETTER_H;
				wstr.insert(i,L"c");
				i++;
				break;
			
			case CYRILLIC_SMALL_LETTER_TSE:
				if ((wstr[i+1]==CYRILLIC_SMALL_LETTER_SOFT_SIGN) || (wstr[i+1]==CYRILLIC_CAPITAL_LETTER_SOFT_SIGN)) 
				{
					wstr[i] = LATIN_SMALL_LETTER_C_WITH_ACUTE;
					wstr.erase((i+1),1);
					break;
				}
				else 
				{
					wstr[i] = LATIN_SMALL_LETTER_C;
					break;
				}
			
			case CYRILLIC_SMALL_LETTER_CHE:
				wstr[i] = LATIN_SMALL_LETTER_C_WITH_CARON;
				break;
			
			case CYRILLIC_SMALL_LETTER_SHA:
				wstr[i] = LATIN_SMALL_LETTER_S_WITH_CARON;
				break;

			case CYRILLIC_SMALL_LETTER_SHCHA:
				wstr[i] = LATIN_SMALL_LETTER_C_WITH_CARON;
				wstr.insert(i, L"\x0161");
				i++;
				break;

			
			case CYRILLIC_SMALL_LETTER_YERU:
				wstr[i] = LATIN_SMALL_LETTER_Y;
				break;

			case CYRILLIC_SMALL_LETTER_SOFT_SIGN:
				wstr[i] = APOSTROPHE;
				break;

			case CYRILLIC_SMALL_LETTER_E:
				wstr[i] = LATIN_SMALL_LETTER_E;
				break;

			case CYRILLIC_SMALL_LETTER_YU:
				// is in the beginning of a word? 
				if ((wstr[i-1] == SPACE) || (wstr[i-1] == CARRIAGE_RETURN) || (wstr[i-1] == 0)) 
				{
					wstr[i] = LATIN_SMALL_LETTER_U;
					wstr.insert(i, L"j");
					i++;
				}

				// preceeded by a vowel?
				else if 
				((wstr[i-1]==LATIN_SMALL_LETTER_A) || (wstr[i-1]==LATIN_SMALL_LETTER_E) || (wstr[i-1]==LATIN_SMALL_LETTER_O) || 
				( wstr[i-1]==LATIN_SMALL_LETTER_I) || (wstr[i-1]==LATIN_SMALL_LETTER_U) || (wstr[i-1]==LATIN_SMALL_LETTER_Y) || 
				(wstr[i-1]==0))
				{
					{wstr[i] = LATIN_SMALL_LETTER_U;}
					wstr.insert(i, L"j");
					i++;
				}
				// Before 'l' ?
				else if (wstr[i-1] == LATIN_SMALL_LETTER_L) 
				{
						wstr[i] = LATIN_SMALL_LETTER_U;
				}
				// Otherwise
				else
				{
					wstr[i] = LATIN_SMALL_LETTER_U;
					wstr.insert(i, L"i");
					i++;
				}
				break;

			case CYRILLIC_SMALL_LETTER_YA:
				// is in the beginning of a word? 
				if ((wstr[i-1] == SPACE) || (wstr[i-1] == CARRIAGE_RETURN) || (wstr[i-1] == 0)) 
				{
					wstr[i] = LATIN_SMALL_LETTER_A;
					wstr.insert(i, L"j");
					i++;
				}

				// preceeded by a vowel?
				else if 
				((wstr[i-1]==LATIN_SMALL_LETTER_A) || (wstr[i-1]==LATIN_SMALL_LETTER_E) || (wstr[i-1]==LATIN_SMALL_LETTER_O) || 
				( wstr[i-1]==LATIN_SMALL_LETTER_I) || (wstr[i-1]==LATIN_SMALL_LETTER_U) || (wstr[i-1]==LATIN_SMALL_LETTER_Y) || 
				(wstr[i-1]==0))
				{
					{wstr[i] = LATIN_SMALL_LETTER_A;}
					wstr.insert(i, L"j");
					i++;
				}
				// Before 'l' ?
				else if (wstr[i-1] == LATIN_SMALL_LETTER_L) 
				{
						wstr[i] = LATIN_SMALL_LETTER_A;
				}
				// Otherwise
				else
				{
					wstr[i] = LATIN_SMALL_LETTER_A;
					wstr.insert(i, L"i");
					i++;
				}
				break;

			}

	}
	if (wbuflen >= wcslen(wstr.c_str()))
	{
		//wcscpy(wbuffer,wstr.c_str());
		//pwstring = wstr.c_str();
		//while(((*wbuffer++)=(*pwstring++))!=(L'\0'));
		wDestination = wbuffer;
		wSource = wstr.c_str();
		while((*wDestination++ = *wSource++) != 0);
		// insert exception
	}

}

void CConvert::L2C(wchar_t* wbuffer, int wbuflen)
{

	wstr = wbuffer;

	for(i=0;i<= (wstr.size());i++)
		{
			switch(wstr[i])
			{

/*************/
/* Uppercase */
/*************/

			case LATIN_CAPITAL_LETTER_A:
				wstr[i] = CYRILLIC_CAPITAL_LETTER_A;
				break;

			case LATIN_CAPITAL_LETTER_B:
				wstr[i] = CYRILLIC_CAPITAL_LETTER_BE;
				break;

			case LATIN_CAPITAL_LETTER_V:
				wstr[i] = CYRILLIC_CAPITAL_LETTER_VE;
				break;

			case LATIN_CAPITAL_LETTER_G:
				wstr[i] = CYRILLIC_CAPITAL_LETTER_GHE;
				break;

			case LATIN_CAPITAL_LETTER_D:
				wstr[i] = CYRILLIC_CAPITAL_LETTER_DE;
				break;

			case LATIN_CAPITAL_LETTER_Z_WITH_CARON:
				wstr[i] = CYRILLIC_CAPITAL_LETTER_ZHE;
				break;

			case LATIN_CAPITAL_LETTER_Z:
				wstr[i] = CYRILLIC_CAPITAL_LETTER_ZE;
				break;

			case LATIN_CAPITAL_LETTER_Z_WITH_ACUTE: //  and 
// Followed by a_lat_lcase or uppercase
				for (x = 0; x <= 31; x ++)
				{
					if (wstr[i+1] == a_lat_lcase[x])
					{
						blcase = true;
						break;
					}						
				}
				if (blcase == true ) 
					wstr[i] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
				else
					wstr[i] = CYRILLIC_CAPITAL_LETTER_SOFT_SIGN;
				wstr.insert(i, L"");
				i++;
				break;

			case LATIN_CAPITAL_LETTER_I:
				if (wstr[i+1] == LATIN_CAPITAL_LETTER_E) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_IE;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_CAPITAL_LETTER_O) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_IO;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_CAPITAL_LETTER_U) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_YU;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_CAPITAL_LETTER_A)  // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_YA;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_E) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_IE;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_O) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_IO;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_U) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_YU;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_A)  // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_YA;
					wstr.erase((i+1),1);
				}

				else 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_BYELORUSSIAN_UKRAINIAN_I;
				}
				break;

			case LATIN_CAPITAL_LETTER_J:
				if (wstr[i+1] == LATIN_CAPITAL_LETTER_E) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_IE;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_CAPITAL_LETTER_O) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_IO;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_CAPITAL_LETTER_U) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_YU;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_CAPITAL_LETTER_A) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_YA;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_E) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_IE;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_O) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_IO;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_U) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_YU;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_A) // cyrillic 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_YA;
					wstr.erase((i+1),1);
				}

				else 
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_SHORT_I;
				}
				break;
			
			case LATIN_CAPITAL_LETTER_K:
				wstr[i] = CYRILLIC_CAPITAL_LETTER_KA;
				break;
			
			case LATIN_CAPITAL_LETTER_L_WITH_STROKE: // L z hacykam
				wstr[i] = CYRILLIC_CAPITAL_LETTER_EL;
				break;

			case LATIN_CAPITAL_LETTER_L: 
// Followed by a_lat_lcase or uppercase
				for (x = 0; x <= 31; x ++)
				{
					if (wstr[i+1] == a_lat_lcase[x])
					{
						blcase = true;
						break;
					}						
				}

				if (wstr[i+1]==SPACE)
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_SOFT_SIGN;
					wstr.insert(i, L"");
					i++;
				}
				else if (wstr[i+1]==LATIN_CAPITAL_LETTER_E)
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_EL;
					wstr[i+1] = CYRILLIC_CAPITAL_LETTER_IE;
					i++;
				}
				else if (wstr[i+1]==LATIN_CAPITAL_LETTER_O)
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_EL;
					wstr[i+1] = CYRILLIC_CAPITAL_LETTER_IO;
					i++;
				}
				else if (wstr[i+1]==LATIN_CAPITAL_LETTER_U)
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_EL;
					wstr[i+1] = CYRILLIC_CAPITAL_LETTER_YU;
					i++;
				}
				else if (wstr[i+1]==LATIN_CAPITAL_LETTER_A)
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_EL;
					wstr[i+1] = CYRILLIC_CAPITAL_LETTER_YA;
					i++;
				}
				else if (wstr[i+1]==LATIN_SMALL_LETTER_E)
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_EL;
					wstr[i+1] = CYRILLIC_SMALL_LETTER_IE;
					i++;
				}
				else if (wstr[i+1]==LATIN_SMALL_LETTER_O)
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_EL;
					wstr[i+1] = CYRILLIC_SMALL_LETTER_IO;
					i++;
				}
				else if (wstr[i+1]==LATIN_SMALL_LETTER_U)
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_EL;
					wstr[i+1] = CYRILLIC_SMALL_LETTER_YU;
					i++;
				}
				else if (wstr[i+1]==LATIN_SMALL_LETTER_A)
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_EL;
					wstr[i+1] = CYRILLIC_SMALL_LETTER_YA;
					i++;
				}
				else if ((wstr[i+1]==LATIN_CAPITAL_LETTER_I) || (wstr[i+1]==LATIN_SMALL_LETTER_I))
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_EL;
				}
				else
				{
					if (blcase == true ) 
					{
						wstr[i] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
						wstr.insert(i, L"");
					}
					else
					{
						wstr[i] = CYRILLIC_CAPITAL_LETTER_SOFT_SIGN;
						wstr.insert(i, L"");
					}

				}
				blcase = false;
				break;

			case LATIN_CAPITAL_LETTER_M:
				wstr[i] = CYRILLIC_CAPITAL_LETTER_EM;
				break;

			case LATIN_CAPITAL_LETTER_N:
				wstr[i] = CYRILLIC_CAPITAL_LETTER_EN;
				break;

			case LATIN_CAPITAL_LETTER_N_WITH_ACUTE: // N z hacykam
// Followed by a_lat_lcase or uppercase
				for (x = 0; x <= 31; x ++)
				{
					if (wstr[i+1] == a_lat_lcase[x])
					{
						blcase = true;
						break;
					}						
				}
				if (blcase == true ) 
					wstr[i] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
				else
					wstr[i] = CYRILLIC_CAPITAL_LETTER_SOFT_SIGN;
				
				wstr.insert(i, L"");
				i++;
				break;

			case LATIN_CAPITAL_LETTER_O: 
				wstr[i] = CYRILLIC_CAPITAL_LETTER_O;
				break;

			case LATIN_CAPITAL_LETTER_P: 
				wstr[i] = CYRILLIC_CAPITAL_LETTER_PE;
				break;

			case LATIN_CAPITAL_LETTER_R: 
				wstr[i] = CYRILLIC_CAPITAL_LETTER_ER;
				break;

			case LATIN_CAPITAL_LETTER_S: 
				wstr[i] = CYRILLIC_CAPITAL_LETTER_ES;
				break;

			case LATIN_CAPITAL_LETTER_S_WITH_ACUTE: // C z hacykam
// Followed by a_lat_lcase or uppercase
				for (x = 0; x <= 31; x ++)
				{
					if (wstr[i+1] == a_lat_lcase[x])
					{
						blcase = true;
						break;
					}						
				}
				if (blcase == true ) 
					wstr[i] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
				else
					wstr[i] = CYRILLIC_CAPITAL_LETTER_SOFT_SIGN;

				wstr.insert(i, L"");
				i++;
				break;

			case LATIN_CAPITAL_LETTER_T: 
				wstr[i] = CYRILLIC_CAPITAL_LETTER_TE;
				break;

			case LATIN_CAPITAL_LETTER_U: 
				wstr[i] = CYRILLIC_CAPITAL_LETTER_U;
				break;

			case LATIN_CAPITAL_LETTER_U_WITH_BREVE: // U z hacykam
				wstr[i] = CYRILLIC_CAPITAL_LETTER_SHORT_U;
				break;

			case LATIN_CAPITAL_LETTER_F: 
				wstr[i] = CYRILLIC_CAPITAL_LETTER_EF;
				break;


			case LATIN_CAPITAL_LETTER_C: //Should be before 'H'
				if (wstr[i+1] == LATIN_CAPITAL_LETTER_H || wstr[i+1] == LATIN_SMALL_LETTER_H)
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_HA;
					wstr.erase((i+1),1);
				}
				else
				{
					wstr[i] = CYRILLIC_CAPITAL_LETTER_TSE;				
				}
				break;

			case LATIN_CAPITAL_LETTER_H: 
				wstr[i] = CYRILLIC_CAPITAL_LETTER_GHE;
				break;


			case LATIN_CAPITAL_LETTER_C_WITH_ACUTE: // C z hacykam
// Followed by a_lat_lcase or uppercase
				for (x = 0; x <= 31; x ++)
				{
					if (wstr[i+1] == a_lat_lcase[x])
					{
						blcase = true;
						break;
					}						
				}
				if (blcase == true ) 
					wstr[i] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
				else
					wstr[i] = CYRILLIC_CAPITAL_LETTER_SOFT_SIGN;

				wstr.insert(i, L"");
				i++;
				break;

			case LATIN_CAPITAL_LETTER_C_WITH_CARON: 
				wstr[i] = CYRILLIC_CAPITAL_LETTER_CHE;
				break;

			case LATIN_CAPITAL_LETTER_S_WITH_CARON: 
				wstr[i] = CYRILLIC_CAPITAL_LETTER_SHA;
				break;

			case LATIN_CAPITAL_LETTER_Y: 
				wstr[i] = CYRILLIC_CAPITAL_LETTER_YERU;
				break;

			case LATIN_CAPITAL_LETTER_E: 
				wstr[i] = CYRILLIC_CAPITAL_LETTER_E;
				break;

/*************/
/* Lowercase */
/*************/

			case LATIN_SMALL_LETTER_A:
				wstr[i] = CYRILLIC_SMALL_LETTER_A;
				break;

			case LATIN_SMALL_LETTER_B:
				wstr[i] = CYRILLIC_SMALL_LETTER_BE;
				break;

			case LATIN_SMALL_LETTER_V:
				wstr[i] = CYRILLIC_SMALL_LETTER_VE;
				break;

			case LATIN_SMALL_LETTER_G:
				wstr[i] = CYRILLIC_SMALL_LETTER_GHE;
				break;

			case LATIN_SMALL_LETTER_D:
				wstr[i] = CYRILLIC_SMALL_LETTER_DE;
				break;

			case LATIN_SMALL_LETTER_Z_WITH_CARON:
				wstr[i] = CYRILLIC_SMALL_LETTER_ZHE;
				break;

			case LATIN_SMALL_LETTER_Z:
				wstr[i] = CYRILLIC_SMALL_LETTER_ZE;
				break;

			case LATIN_SMALL_LETTER_Z_WITH_ACUTE: // z z hacykam
				wstr[i] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
				wstr.insert(i, L"");
				i++;
				break;

			case LATIN_SMALL_LETTER_I:
				if (wstr[i+1] == LATIN_SMALL_LETTER_E) // cyrillic 
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_IE;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_O) // cyrillic 
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_IO;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_U) // cyrillic 
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_YU;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_A)  // cyrillic 
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_YA;
					wstr.erase((i+1),1);
				}

				else 
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_BYELORUSSIAN_UKRAINIAN_I;
				}
				break;

			case LATIN_SMALL_LETTER_J:
				if (wstr[i+1] == LATIN_SMALL_LETTER_E) // cyrillic 
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_IE;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_O) // cyrillic 
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_IO;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_U) // cyrillic 
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_YU;
					wstr.erase((i+1),1);
				}
				else if (wstr[i+1] == LATIN_SMALL_LETTER_A) // cyrillic 
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_YA;
					wstr.erase((i+1),1);
				}

				else 
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_SHORT_I;
				}
				break;
			
			case LATIN_SMALL_LETTER_K:
				wstr[i] = CYRILLIC_SMALL_LETTER_KA;
				break;
			
			case LATIN_SMALL_LETTER_L_WITH_STROKE: // l z hacykam
				wstr[i] = CYRILLIC_SMALL_LETTER_EL;
				break;

			case LATIN_SMALL_LETTER_L: 
				if (wstr[i+1]==SPACE)
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
					wstr.insert(i, L"");
					i++;
				}
				else if (wstr[i+1]==LATIN_SMALL_LETTER_E)
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_EL;
					wstr[i+1] = CYRILLIC_SMALL_LETTER_IE;
					i++;
				}
				else if (wstr[i+1]==LATIN_SMALL_LETTER_O)
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_EL;
					wstr[i+1] = CYRILLIC_SMALL_LETTER_IO;
					i++;
				}
				else if (wstr[i+1]==LATIN_SMALL_LETTER_U)
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_EL;
					wstr[i+1] = CYRILLIC_SMALL_LETTER_YU;
					i++;
				}
				else if (wstr[i+1]==LATIN_SMALL_LETTER_A)
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_EL;
					wstr[i+1] = CYRILLIC_SMALL_LETTER_YA;
					i++;
				}
				else if (wstr[i+1]==LATIN_SMALL_LETTER_I)
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_EL;
				}
				else
				{
				wstr[i] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
				wstr.insert(i, L"");
				}
				break;

			case LATIN_SMALL_LETTER_M:
				wstr[i] = CYRILLIC_SMALL_LETTER_EM;
				break;

			case LATIN_SMALL_LETTER_N:
				wstr[i] = CYRILLIC_SMALL_LETTER_EN;
				break;

			case LATIN_SMALL_LETTER_N_WITH_ACUTE: // N z hacykam
				wstr[i] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
				wstr.insert(i, L"");
				i++;
				break;

			case LATIN_SMALL_LETTER_O: 
				wstr[i] = CYRILLIC_SMALL_LETTER_O;
				break;

			case LATIN_SMALL_LETTER_P: 
				wstr[i] = CYRILLIC_SMALL_LETTER_PE;
				break;

			case LATIN_SMALL_LETTER_R: 
				wstr[i] = CYRILLIC_SMALL_LETTER_ER;
				break;

			case LATIN_SMALL_LETTER_S: 
				wstr[i] = CYRILLIC_SMALL_LETTER_ES;
				break;

			case LATIN_SMALL_LETTER_S_WITH_ACUTE: // C z hacykam
				wstr[i] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
				wstr.insert(i, L"");
				i++;
				break;

			case LATIN_SMALL_LETTER_T: 
				wstr[i] = CYRILLIC_SMALL_LETTER_TE;
				break;

			case LATIN_SMALL_LETTER_U: 
				wstr[i] = CYRILLIC_SMALL_LETTER_U;
				break;

			case LATIN_SMALL_LETTER_U_WITH_BREVE: // U z hacykam
				wstr[i] = CYRILLIC_SMALL_LETTER_SHORT_U;
				break;

			case LATIN_SMALL_LETTER_F: 
				wstr[i] = CYRILLIC_SMALL_LETTER_EF;
				break;

			case LATIN_SMALL_LETTER_C: //Should be before 'h'
				if (wstr[i+1] == LATIN_CAPITAL_LETTER_H || wstr[i+1] == LATIN_SMALL_LETTER_H)
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_HA;
					wstr.erase((i+1),1);
				}
				else
				{
					wstr[i] = CYRILLIC_SMALL_LETTER_TSE;				
				}
				break;

			case LATIN_SMALL_LETTER_H: 
				wstr[i] = CYRILLIC_SMALL_LETTER_GHE;
				break;

			case LATIN_SMALL_LETTER_C_WITH_ACUTE: // C z hacykam
				wstr[i] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
				wstr.insert(i, L"");
				i++;
				break;

			case LATIN_SMALL_LETTER_C_WITH_CARON: 
				wstr[i] = CYRILLIC_SMALL_LETTER_CHE;
				break;

			case LATIN_SMALL_LETTER_S_WITH_CARON: 
				wstr[i] = CYRILLIC_SMALL_LETTER_SHA;
				break;

			case LATIN_SMALL_LETTER_Y: 
				wstr[i] = CYRILLIC_SMALL_LETTER_YERU;
				break;

			case LATIN_SMALL_LETTER_E: 
				wstr[i] = CYRILLIC_SMALL_LETTER_E;
				break;

// In case russian text occurs
/*			case L'\'': 
				wstr[i] = CYRILLIC_SMALL_LETTER_SOFT_SIGN;
				break;
*/
			}
	}

	if (wbuflen >= wcslen(wstr.c_str()))
	{
		//wcscpy(wbuffer,wstr.c_str());
		//pwstring = wstr.c_str();
		//while(((*wbuffer++)=(*pwstring++))!=(L'\0'));
		wDestination = wbuffer;
		wSource = wstr.c_str();
		while((*wDestination++ = *wSource++) != 0);
		// insert exception
	}
}

CConvert::~CConvert(void)
{


}

