/*
trayicon.exe - WIN32 application which resides in the system tray
Copyright (C) 2000  Antti Markus (antti@pld.ttu.ee)

Program that convers Belarusian text to/from kyrylica and lacinka

Copyright (C) 2000  Alexander Mikhailian (mikhailian@altern.org)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __GNUC__
//useful defines
#define STRICT
#define WIN32_LEAN_AND_MEAN
#endif

#ifndef _UNICODE
#define _UNICODE
#endif

//standard includes
#include <windows.h>

//this is not defined in mingw32 versions prior to 2.95
#ifndef TPM_BOTTOMALIGN
#define TPM_BOTTOMALIGN     0x0020L
#endif


//additional includes
#include "trayicon.h"
#include "convert.h"
#include "resource.h"

/////////////////////////////////////////////////////////////////////////////
// Cross-compilation - should be moved to a separate file soon
// GCC has no wcscpy
//#ifdef __GNUC__
//	wcscpy((wchar_t*)wbuffer,(const wchar_t*)hglb); 
//	while((*(wchar_t*)wbuffer++=*(const wchar_t*)hglb)!=(L'\0'));
//#endif
// End cross-compilation
//////////////////////////////////////////////////////////////////


//trayicon notification message
#define WM_TRAY   WM_USER+0x100+5

LRESULT CALLBACK WindowFunc(HWND, UINT, WPARAM, LPARAM);

const char* pClassName="TTWin32TrayApp123"; //window class name
HINSTANCE hMainInst=0;    //application main instance handle
HICON hIcon1=0, hIcon2=0; //icon handles
unsigned uWhich=0;        //indicates which icon is displayed
HMENU hPopup=0;           //menu handle
CTrayIcon* pTrayIcon=0;   //pointer to trayicon class
// CLipboard-related
CConvert* pConvert=0;			//pointer to Convert class
//HGLOBAL   hglb; // contents of the clipboard
wchar_t*   hglb; // contents of the clipboard
wchar_t*  whglb; // contents of the clipboard as wchar_t
//char*     ahglb; // contents of the clipboard as char
HGLOBAL		clipbuffer;
HGLOBAL		clipwbuffer;
char*		buffer;
wchar_t*	wbuffer;
int			buflen;
int			wbuflen;
const int KEY_T = 001;
const int KEY_L = 002;
const int KEY_C = 003;

int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPSTR lpszArgs,
                   int nWinMode)
{
  WNDCLASS WinClass;

  //register window class
  WinClass.hInstance=hInstance;
  hMainInst=hInstance;
  WinClass.lpszClassName=pClassName;  //specify class name
  WinClass.lpfnWndProc=WindowFunc;    //specify callback function
  WinClass.style=0;
  WinClass.hIcon=LoadIcon(NULL, IDI_APPLICATION); //specify icon
  WinClass.hCursor=LoadCursor(NULL, IDC_ARROW);   //specify cursor
  WinClass.lpszMenuName=0;
  WinClass.cbClsExtra=0;
  WinClass.cbWndExtra=0;
  WinClass.hbrBackground=(HBRUSH)GetStockObject(WHITE_BRUSH); //specify window background

  if(!RegisterClass(&WinClass))
    return 0;

  //create window
  HWND hwnd=CreateWindow(pClassName,
                         0,
                         WS_OVERLAPPEDWINDOW,
                         10,
                         10,
                         10,
                         10,
                         HWND_DESKTOP,
                         NULL,
                         hInstance,
                         NULL);

  //hide window
  ShowWindow(hwnd, SW_HIDE);

  // Register system-wide HotKey
  RegisterHotKey(hwnd, KEY_T,  MOD_ALT | MOD_CONTROL, 'T'); //T
  RegisterHotKey(hwnd, KEY_L,  MOD_ALT | MOD_CONTROL, 'L'); //L
  RegisterHotKey(hwnd, KEY_C,  MOD_ALT | MOD_CONTROL, 'C'); //C

  //start message pump
  MSG WinMsg;
  while(GetMessage(&WinMsg, NULL, 0, 0))
  {
    TranslateMessage(&WinMsg);
    DispatchMessage(&WinMsg);
  }

  return WinMsg.wParam;
}

LRESULT CALLBACK WindowFunc(HWND hwnd, UINT Msg,
                            WPARAM wParam, LPARAM lParam)
{
  switch(Msg)
  {
//////////////////////////////////////////////////////
//                                                  //
//              Application starts up               //
//                                                  //
//////////////////////////////////////////////////////
    case WM_CREATE:
    {
      //initialize icons
      hIcon1=LoadIcon(hMainInst, MAKEINTRESOURCE(IDI_CHECK));
      //hIcon2=LoadIcon(hMainInst, MAKEINTRESOURCE(IDI_CROSS));
      //initialize popup menu
      hPopup=LoadMenu(hMainInst, MAKEINTRESOURCE(IDR_MENU));

      //initialize tray icon
      pTrayIcon=new CTrayIcon(hwnd, WM_TRAY, IDI_CHECK, hIcon1, "Ready");
      //put icon to system tray
      pTrayIcon->Put();
      
			// initialize Convert class
			pConvert=new CConvert();
	  
	  break;
    }
	//////////////////////////////////////////////////////
	//                                                  //
	//                   HotKey pressed                 //
	//                                                  //
	//////////////////////////////////////////////////////

	case WM_HOTKEY:
	{
      switch(LOWORD(wParam))
      {
		case KEY_T:
		{
			while (!PostMessage(hwnd,WM_COMMAND,IDM_C2T,0));
			break;
		}
		case KEY_L:
		{
			while (!PostMessage(hwnd,WM_COMMAND,IDM_C2L,0));
			break;
		}
		case KEY_C:
		{
			while (!PostMessage(hwnd,WM_COMMAND,IDM_L2C,0));
			break;
		}
	  }
	}


	//////////////////////////////////////////////////////
	//                                                  //
	//      Notification fro Trayicon Received          //
	//                                                  //
	//////////////////////////////////////////////////////
    case WM_TRAY:
    {
      //if right mouse button pressed
      if(lParam == WM_RBUTTONDOWN)
      {
        POINT pt;
        GetCursorPos(&pt);  //get cursor position
        SetForegroundWindow(hwnd);  //set window to foreground
        //display popup menu
        BOOL bRes=TrackPopupMenu(GetSubMenu(hPopup, 0), TPM_BOTTOMALIGN | TPM_RIGHTALIGN,
                                 pt.x, pt.y, 0, hwnd, 0);
        //send dummy message to window
        SendMessage(hwnd, WM_NULL, 0, 0);
      }
      break;
    }

	//////////////////////////////////////////////////////
	//                                                  //
	//         Notification from menu received          //
	//                                                  //
	//////////////////////////////////////////////////////
    case WM_COMMAND:
    {
      switch(LOWORD(wParam))
      {
        case IDM_EXIT:
        {
          DestroyWindow(hwnd);
          break;
        }
		//////////////////////////////////////////////////////
		//                                                  //
		//      Convert Cyrillic to Transliteration         //
		//                                                  //
		//////////////////////////////////////////////////////
        case IDM_C2T:
        {
			/* First see if CF_UNICODETEXT is available. if yes -
			process UNICODETEXT, */
			if (IsClipboardFormatAvailable(CF_UNICODETEXT) != (bool)NULL) 
			{
				if (OpenClipboard(hwnd) != (bool)NULL) 
				{
					hglb = (wchar_t*)GetClipboardData(CF_UNICODETEXT); 
					if (hglb != NULL) 
					{			
						GlobalLock(hglb);
						if ((buflen = wcslen(hglb)) != (bool)NULL)
						{
							// Allocate memory for the wide string
							clipwbuffer = GlobalAlloc(GMEM_MOVEABLE, (buflen * 4));
							wbuffer = (wchar_t*)GlobalLock(clipwbuffer);
							
							// Allocate memory for the simple string
							clipbuffer = GlobalAlloc(GMEM_MOVEABLE, (buflen * 2));
							buffer = (char*)GlobalLock(clipbuffer);
							// Copy CF_UNICODETEXT into WideString
							//wcscpy((wchar_t*)wbuffer, (const wchar_t*)hglb);
							//whglb = (wchar_t*)hglb;
							whglb = wbuffer;
							while((*whglb++ = *hglb++) != L'\0');

							EmptyClipboard();
							GlobalUnlock(hglb);
							// The actual conversion
							pConvert->C2T((wchar_t*)wbuffer, buflen * 2);
																				
							GlobalUnlock(clipwbuffer);
							GlobalUnlock(clipbuffer);

							// Copy wbuffer to buffer 
							WideCharToMultiByte( 1251, 0, 
								wbuffer, wcslen(wbuffer), 
								 buffer, strlen( buffer), 
								 NULL, 
								 NULL);

							if  (
								(SetClipboardData(CF_UNICODETEXT,clipwbuffer) != (bool)NULL)
								||
								(SetClipboardData(CF_TEXT,clipbuffer) != (bool)NULL)
								)
								pTrayIcon->Modify("Converted Cyrillic to Transliteration");
							else
								pTrayIcon->Modify("Failed to set Clipboard data");
						}

					}
				}
				CloseClipboard();				
			}
			/* Then if only CF_TEXT is available - process CF_TEXT, */			
			else if (IsClipboardFormatAvailable(CF_TEXT) != (bool)NULL) 
			{
				if (OpenClipboard(hwnd) != (bool)NULL) 
				{
					hglb = (wchar_t*)GetClipboardData(CF_TEXT); 
					if (hglb != (bool)NULL) 
					{			
						GlobalLock(hglb);

						if ((buflen = strlen((const char*)hglb)) != (bool)NULL)
						{
							// Allocate memory for the wide string
							clipwbuffer = GlobalAlloc(GMEM_MOVEABLE, (buflen * 4));
							wbuffer = (wchar_t*)GlobalLock(clipwbuffer);
							
							// Allocate memory for the simple string
							clipbuffer = GlobalAlloc(GMEM_MOVEABLE, (buflen * 2));
							buffer = (char*)GlobalLock(clipbuffer);

							strcpy((char*)buffer, (const char*)hglb);

							// Copy buffer to wbuffer 
							MultiByteToWideChar( 1251, 0, 
								buffer, strlen( buffer), 
								wbuffer, wcslen(wbuffer));

							EmptyClipboard();
							GlobalUnlock(hglb);
							
							pConvert->C2T((wchar_t*)wbuffer, buflen * 2);

							// Copy wbuffer back to buffer 
							WideCharToMultiByte( 1251, 0, 
								wbuffer, wcslen(wbuffer), 
								 buffer, strlen( buffer), 
								 NULL, NULL);
																				
							GlobalUnlock(clipwbuffer);
							GlobalUnlock(clipbuffer);

							if  (SetClipboardData(CF_TEXT,clipbuffer) != (bool)NULL)								//)
								pTrayIcon->Modify("Converted Cyrillic to Transliteration");
							else
								pTrayIcon->Modify("failed to set Clipboard data");
						}

					}
				}
				CloseClipboard();
			} 


			else //IsClipboardFormatAvailable(CF_TEXT) != NULL
				pTrayIcon->Modify("No Clipboard data available");

          	break;
        }


		//////////////////////////////////////////////////////
		//                                                  //
		//          Convert Cyrillic to Lacinka             //
		//                                                  //
		//////////////////////////////////////////////////////
        case IDM_C2L:
        {
			if (IsClipboardFormatAvailable(CF_UNICODETEXT) != (bool)NULL) 
			{
				if (OpenClipboard(hwnd) != (bool)NULL) 
				{
					hglb = (wchar_t*)GetClipboardData(CF_UNICODETEXT); 
					if (hglb != (bool)NULL) 
					{			
						GlobalLock(hglb);
						if ((buflen = wcslen((const wchar_t*)hglb)) != (bool)NULL)
						{
							clipbuffer = GlobalAlloc(GMEM_MOVEABLE, (buflen * 4));
							wbuffer = (wchar_t*)GlobalLock(clipbuffer);
							//wcscpy((wchar_t*)wbuffer, (const wchar_t*)hglb);
							//whglb = (wchar_t*)hglb;
							//while(((*wbuffer++)=(*whglb++))!=(L'\0'));
							whglb = wbuffer;
							while((*whglb++ = *hglb++) != L'\0');
							
							EmptyClipboard();
							GlobalUnlock(hglb);
						
							pConvert->C2L((wchar_t*)wbuffer, buflen * 2);
																				
							GlobalUnlock(clipbuffer);

							if(SetClipboardData(CF_UNICODETEXT,clipbuffer) != NULL)
								pTrayIcon->Modify("Converted Cyrillic to Lacinka");
							else
								pTrayIcon->Modify("Failed to set Clipboard data");
						}

					}
				}
				CloseClipboard();
			}
			else //IsClipboardFormatAvailable(CF_UNICODETEXT) != NULL
				pTrayIcon->Modify("No Clipboard data available");

			break;
        }

		//////////////////////////////////////////////////////
		//                                                  //
		//          Convert Lacinka to Cyrillic             //
		//                                                  //
		//////////////////////////////////////////////////////
		case IDM_L2C:
		{
			if (IsClipboardFormatAvailable(CF_UNICODETEXT) != (bool)NULL) 
			{
				if (OpenClipboard(hwnd) != (bool)NULL) 
				{
					hglb = (wchar_t*)GetClipboardData(CF_UNICODETEXT); 
					if (hglb != (bool)NULL) 
					{			
						GlobalLock(hglb);
						if ((buflen = wcslen((const wchar_t*)hglb)) != (bool)NULL)
						{
							clipbuffer = GlobalAlloc(GMEM_MOVEABLE, (buflen * 4));
							wbuffer = (wchar_t*)GlobalLock(clipbuffer);
							//scpy((wchar_t*)wbuffer, (const wchar_t*)hglb);
							//whglb = (wchar_t*)hglb;
							//while(((*wbuffer++)=(*whglb++))!=(L'\0'));
							whglb = wbuffer;
							while((*whglb++ = *hglb++) != L'\0');

							EmptyClipboard();
							GlobalUnlock(hglb);
						
							pConvert->L2C((wchar_t*)wbuffer, buflen * 2);
																				
							GlobalUnlock(clipbuffer);

							if(SetClipboardData(CF_UNICODETEXT,clipbuffer) != (bool)NULL)
								pTrayIcon->Modify("Converted Lacinka to Cyrillic");
							else
								pTrayIcon->Modify("Failed to set Clipboard data");
						}

					}
				}
				CloseClipboard();
			}
			else //IsClipboardFormatAvailable(CF_UNICODETEXT) != NULL
				pTrayIcon->Modify("No Clipboard data available");

			break;
		}

		//////////////////////////////////////////////////////
		//                                                  //
		//        Convert Lacinka to Transliteration        //
		//                                                  //
		//////////////////////////////////////////////////////
		case IDM_L2T:
		{
			if (IsClipboardFormatAvailable(CF_UNICODETEXT) != (bool)NULL) 
			{
				if (OpenClipboard(hwnd) != (bool)NULL) 
				{
					hglb = (wchar_t*)GetClipboardData(CF_UNICODETEXT); 
					if (hglb != (bool)NULL) 
					{			
						GlobalLock(hglb);
						if ((buflen = wcslen((const wchar_t*)hglb)) != (bool)NULL)
						{
							clipbuffer = GlobalAlloc(GMEM_MOVEABLE, (buflen * 4));
							wbuffer = (wchar_t*)GlobalLock(clipbuffer);
							//scpy((wchar_t*)wbuffer, (const wchar_t*)hglb);
							//whglb = (wchar_t*)hglb;
							//while(((*wbuffer++)=(*whglb++))!=(L'\0'));
							whglb = wbuffer;
							while((*whglb++ = *hglb++) != L'\0');

							EmptyClipboard();
							GlobalUnlock(hglb);
						
							pConvert->L2C((wchar_t*)wbuffer, buflen * 2);
							pConvert->C2T((wchar_t*)wbuffer, buflen * 2);
																				
							GlobalUnlock(clipbuffer);

							if(SetClipboardData(CF_UNICODETEXT,clipbuffer) != (bool)NULL)
								pTrayIcon->Modify("Converted Lacinka to Transliteration");
							else
								pTrayIcon->Modify("Failed to set Clipboard data");
						}

					}
				}
				CloseClipboard();
			}
			else //IsClipboardFormatAvailable(CF_UNICODETEXT) != NULL
				pTrayIcon->Modify("No Clipboard data available");

			break;
		}



		//////////////////////////////////////////////////////
		//                                                  //
		//                 Pop up Help dialog               //
		//                                                  //
		//////////////////////////////////////////////////////
		case IDM_HELP:
			{
				MessageBox(hwnd,
"                 USING KIRLAT\n\
\n\
Kirlat  works with  the  clipboard contents.\n\
Just  copy  the   text into   the   clipboard,\n\
perform  one   of    4   conversions   on  it:\n\
\n\
    Lat2Cyr   -  lacinka to kirylica\n\
    (keyboard shortcut - Ctrl+Alt+C)\n\n\
    Lat2Tra   -  lacinka to transliteration\n\
    (no keyboard shortcut)\n\
\n\
    Cyr2Lat   -  kirylica to lacinka\n\
   (keyboard shortcut - Ctrl+Alt+L)\n\
\n\
    Cyr2Tra   -  kirylica to transliteration\n\
    (keyboard shortcut - Ctrl+Alt+T)\n\
\n\
and  then paste  back the contents of  the\n\
clipboard to your application.\n\
\n\
Beware that all extra formatting will be lost",
				"Help!"
				,MB_OK);
				break;
			}

		//////////////////////////////////////////////////////
		//                                                  //
		//             Pop up the About dialog              //
		//                                                  //
		//////////////////////////////////////////////////////
		case IDM_ABOUT:
			{
				MessageBox(hwnd,
"      Converter of Belarusian text to/from kirylica and lacinka\n\
\n\
                    Report bugs to mikhailian@altern.org\n",
/*\
"      Converter of Belarusian text to/from kirylica and lacinka\n\
\n\
     Copyright (C) Alexander Mikhailian <mikhailian@altern.org>\n\
\n\
This program is free software; you can  redistribute it  and/or  modify\n\
it under the terms of  the GNU General Public License as published \n\
by the Free Software Foundation; either version 2 of the License, or\n\
(at your option) any later version.\n\
\n\
This  program  is  distributed  in  the  hope  that  it  will  be useful, but\n\
WITHOUT ANY WARRANTY; without  even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
See the GNU General Public License for more details.\n\
\n\
You should have received a copy of the GNU General Public License\n\
along with this program; if not, write to the  Free Software Foundation,\n\
Inc., 59 Temple Place  -  Suite 330, Boston, MA  02111-1307, USA.\n\
\n\
                    Report bugs to mikhailian@altern.org\n",
*/\
				"About"
				,MB_OK);
				break;
			}

      }
	  break;
	}


	//////////////////////////////////////////////////////
	//                                                  //
	//                Exit application                  //
	//                                                  //
	//////////////////////////////////////////////////////
    case WM_DESTROY:
    {
      //release used resources
      delete pTrayIcon;
      delete pConvert;
      DestroyIcon(hIcon1);
      DestroyIcon(hIcon2);
      DestroyMenu(hPopup);
      PostQuitMessage(0);
      break;
    }

    default:
      return DefWindowProc(hwnd, Msg, wParam, lParam);
  }
  return 0;
}
